/*
 * A registry of filesystem objects.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/directory_provider.h>
#include <fsserver/file_opening.h>
#include <fsserver/file_provider.h>
#include <fsserver/file_registry.h>
#include <fsserver/pager.h>
#include <fsserver/pages.h>



/* A registry of filesystem objects. */

class FileObjectRegistry : public FileRegistry
{
protected:
    Pages *_pages;

    /* Resource initialisation methods. */

    map_flags_t get_flags(flags_t flags);

    long get_directory_provider(FileOpening *opening, const char *path,
                                flags_t flags, fileid_t fileid,
                                DirectoryProvider **directory_provider);

    long get_file_provider(FileOpening *opening, const char *path,
                           flags_t flags, fileid_t fileid,
                           FileProvider **file_provider);

    /* Resource initialisation methods. */

    long get_directory(FileOpening *opening, const char *path, flags_t flags,
                       fileid_t fileid, offset_t *size, object_flags_t *object_flags,
                       Resource **resource);

    long get_file(FileOpening *opening, const char *path, flags_t flags,
                  fileid_t fileid, offset_t *size, object_flags_t *object_flags,
                  Resource **resource);

public:
    explicit FileObjectRegistry(Pages *pages);

    /* Resource discovery methods. */

    long find_directory_provider(fileid_t fileid,
                                 DirectoryProvider **directory_provider);

    /* Resource initialisation methods. */

    long get_resource(FileOpening *opening, const char *path, flags_t flags,
                      offset_t *size, object_flags_t *object_flags,
                      Resource **resource);
};

// vim: tabstop=4 expandtab shiftwidth=4
