/*
 * An object providing access to file functionality.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "file_provider.h"



/* Initialise the provider with a page 'mapper' for the file's contents. */

FileProvider::FileProvider(PageMapper *mapper)
: Provider(), _mapper(mapper)
{
}

/* Deallocate the provider's resources. */

FileProvider::~FileProvider()
{
    /* Accessors are allocated exclusively for page mappers for files, and
       so can be deleted here. In general, the page mapper may not have
       exclusive ownership of an accessor. */

    Accessor *accessor = _mapper->accessor();
    delete _mapper;
    delete accessor;
}

/* Return the page mapper provided. */

PageMapper *FileProvider::mapper()
{
    return _mapper;
}

// vim: tabstop=4 expandtab shiftwidth=4
