/*
 * Base types used by various other types.
 *
 * Copyright (C) 2019, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/compiler.h>

#include <systypes/factory.h>

EXTERN_C_BEGIN

/* unsigned long is a 64-bit type.
   See: pkg/l4re-core/l4sys/include/l4int.h */

/* Types for file access (access flags, offsets) and IPC operations. */

typedef unsigned long       flags_t;
typedef unsigned long       offset_t;
typedef l4_addr_t           address_t;      /* unsigned long */

/* Dataspace mapping flags.
   See: pkg/l4re-core/l4re_c/include/dataspace.h */

typedef unsigned long       map_flags_t;    /* 64-bit */

/* Types and values for notification. */

typedef unsigned long       notify_flags_t;

enum notify_flags
{
  NOTIFY_CONTENT_AVAILABLE  = 1,            /* reading files and pipes */
  NOTIFY_SPACE_AVAILABLE    = 2,            /* writing pipes */
  NOTIFY_PEER_CLOSED        = 4,            /* closing files and pipes */
  NOTIFY_FILE_OPENED        = 8,            /* opening files in directories */
};

/* Filesystem object properties. */

typedef unsigned long       object_flags_t;

enum object_flags
{
  OBJECT_SUPPORTS_MMAP      = 1,
  OBJECT_HAS_SIZE           = 2
};

/* Equivalent types are defined in sys/types.h typically. In newlib, they are
   defined in sys/_types.h if not defined elsewhere (such as in
   machine/_types.h). */

typedef unsigned long long  sys_dev_t;      /* device number */
typedef unsigned long       sys_ino_t;      /* inode number */
typedef unsigned int        sys_mode_t;     /* file permissions */
typedef unsigned int        sys_nlink_t;    /* link count */
typedef unsigned int        sys_uid_t;      /* user identifier */
typedef unsigned int        sys_gid_t;      /* group identifier */
typedef long                sys_off_t;      /* file offset/position */
typedef long                sys_blksize_t;  /* input/output block size */
typedef long                sys_blkcnt_t;   /* number of 512-byte blocks */

/* Factory types for L4 factory interface invocations. */

ipc_varg_typedef(sys_mode_t, ipc_varg_sys_mode_t)
ipc_varg_typedef(sys_uid_t, ipc_varg_sys_uid_t)
ipc_varg_typedef(sys_gid_t, ipc_varg_sys_gid_t)

#define ipc_varg_sys_mode(value) \
ipc_varg_umword(ipc_varg_sys_mode_t, value)

#define ipc_varg_sys_uid(value) \
ipc_varg_umword(ipc_varg_sys_uid_t, value)

#define ipc_varg_sys_gid(value) \
ipc_varg_umword(ipc_varg_sys_gid_t, value)

EXTERN_C_END

// vim: tabstop=2 expandtab shiftwidth=2
